(function() {
	'use strict';

	angular
		.module('barebone.buslist')
		.controller('BuslistController', BuslistController);

	BuslistController.$inject = [
		'busService',
		'$timeout',
		'$scope',
		'$state',
		'$rootScope',
		'$ionicPopup',
		'busResult',
		'$window',
		'$interval'
	];

	/* @ngInject */
	function BuslistController(
		busService, $timeout,$scope,$state,$rootScope,$ionicPopup,busResult,$window,$interval) {

		if($rootScope.stopTimeInt){
				$interval.cancel($rootScope.stopTimeInt);
		}

		$rootScope.$broadcast('loading:hide');
		
		$scope.today = new Date();
		$window.localStorage['dollar_rate'] = busResult.result.dollar_rate;
		
		var searchResult = $rootScope.searchResult;
		var userInfo     = $rootScope.userInfo || {};

		var vm = angular.extend(this, {
			from         : searchResult.from_destination,
			to 			 : searchResult.to_destination,
			userInfo	 : userInfo,
			adult 		 : searchResult.adult,
		traveldate 		 : searchResult.traveldate,
			busLists 	 : busResult.result.buses.data,
			avaliable	 : busResult.result.avaliable,
			busId		 : '',
			dataLoad     : 0,
	before_block_min	 : busResult.result.before_block_min,
			fullyLoad    : true,
			increaseSeat : increaseSeat,
			seatId		 : '',
			page 		 : 1,
			dollar_rate	 : busResult.result.dollar_rate,
			totalPages	 : busResult.result.buses.last_page,
			doRefresh	 : doRefresh,
	moreDataCanBeLoaded	 : moreDataCanBeLoaded,
			checkDate	 : checkDate,
			loadMore	 : loadMore
		});

		(function activate() {
			$rootScope.avaliable = vm.avaliable;
			$rootScope.before_block_min = vm.before_block_min;
			$rootScope.busLists = vm.busLists;
		})();

		function checkDate(travel_date){
			var dtravel_date = travel_date.split('-');
			var min  	=  dtravel_date[3].split(':');

			var departure_date = new Date(dtravel_date[2],dtravel_date[1]-1,dtravel_date[0],min[0],min[1]);

			var current_date = new Date();
			var hours = parseInt(vm.before_block_min/60);
			var minutes = parseInt(vm.before_block_min%60);

			departure_date.setHours(departure_date.getHours() - hours);
			departure_date.setMinutes(departure_date.getMinutes() - minutes);

			if(current_date <= departure_date){
				return true;
			}
			else{
				return false;
			}
		}

		function increaseSeat(){
			vm.dataLoad += 1;
		}

		function moreDataCanBeLoaded(){
			return vm.totalPages > vm.page;
		}

		function doRefresh() {
			vm.page = 1;
			vm.fullyLoad = false;

			busService.getBusroutes(vm.page,searchResult)
				.then(function(data) {
				vm.fullyLoad = true;

					vm.totalPages = data.result.buses.last_page;		
					 vm.busLists = data.result.buses.data;
					 vm.avaliable = data.result.avaliable;
					 $rootScope.avaliable = vm.avaliable;
					 $rootScope.busLists = vm.busLists;
					$scope.$broadcast('scroll.refreshComplete');
				});
		}

		function loadMore() {
			vm.page += 1;
			vm.fullyLoad = false;

			busService.getBusroutes(vm.page,searchResult)
				.then(function(data) {	
				vm.fullyLoad = true;

					vm.totalPages = data.result.buses.last_page;
					var new_busLists = data.result.buses.data;
					vm.busLists = vm.busLists.concat(new_busLists);
					vm.avaliable = data.result.avaliable;
					$rootScope.avaliable = vm.avaliable;
					$rootScope.busLists = vm.busLists;
					$scope.$broadcast('scroll.infiniteScrollComplete');
				});
		}

	}
})();