(function() {
  'use strict';

angular.module('factories', [])

.factory('DBA', function($cordovaSQLite, $q, $ionicPlatform) {
        var self = this;
        self.init = function() {
          try {
                if (window.cordova) {
                    console.log("TRUE");
                    db = $cordovaSQLite.openDB({ name: 'jjexpress.db', location: 'default' });
                } else {
                  console.log("FALSE");

                    db = window.openDatabase("jjexpress.db", "1.0", "JJ Express ", -1);
                }
            } catch (error) {
                alert(error);
                console.log("ERROR");
                console.log(error);

            }
        }
        self.query = function(query, parameters) {
            parameters = parameters || [];
            var q = $q.defer();
            $ionicPlatform.ready(function() {
                $cordovaSQLite.execute(db, query, parameters)
                    .then(function(result) {
                        q.resolve(result);
                    }, function(error) {
                        q.reject(error);
                    });
            });
            return q.promise;
        }
        self.getAll = function(result) {
            var output = [];
            for (var i = 0; i < result.rows.length; i++) {
                output.push(result.rows.item(i));
            }
            return output;
        }
        self.getById = function(result) {
            var output = null;
            output = angular.copy(result.rows.item(0));
            return output;
        }
        return self;
})

.factory('SqlDestinations', function($cordovaSQLite, DBA) {
        var self = this;

        self.initialize = function() {
            return DBA.query("CREATE TABLE IF NOT EXISTS Destinations (id INTEGER PRIMARY KEY AUTOINCREMENT, name TEXT,mm_name TEXT)")
                .then(function(result) {
                    return true;
                });
        }
        self.all = function() {
            return DBA.query("SELECT * FROM Destinations ORDER BY id ASC")
                .then(function(result) {
                    return DBA.getAll(result);
                });
        }
        self.get = function(id) {
            var parameters = [id];
            return DBA.query("SELECT * FROM Destinations WHERE id = (?)", parameters)
                .then(function(result) {
                    return DBA.getById(result);
                });
        }
        self.add = function(obj) {
              var parameters = [obj.id, obj.name, obj.mm_name];
              return DBA.query("INSERT OR REPLACE INTO Destinations (id,name,mm_name) VALUES (?,?,?)", parameters);
        }
        self.remove = function(obj) {
            var parameters = [obj.id];
            return DBA.query("DELETE FROM Destinations WHERE id = (?)", parameters);
        }
        self.update = function(oldid, newDataObj) {
            var parameters = [newDataObj.name, newDataObj.mm_name, oldid];
            return DBA.query("UPDATE Destinations SET name = (?), mm_name = (?) WHERE id = (?)", parameters);
        }
        return self;
})

.factory('ConnectivityMonitor', function($rootScope, $cordovaNetwork,$cordovaToast){
  return {
    isOnline: function(){
      if(ionic.Platform.isWebView()){
        return $cordovaNetwork.isOnline();    
      } else {
        return navigator.onLine;
      }
    },
    isOffline: function(){
      if(ionic.Platform.isWebView()){
        return !$cordovaNetwork.isOnline();    
      } else {
        return !navigator.onLine;
      }
    },
    startWatching: function(){
        if(ionic.Platform.isWebView()){
 
          $rootScope.$on('$cordovaNetwork:online', function(event, networkState){
            console.log("went online");
          });
 
          $rootScope.$on('$cordovaNetwork:offline', function(event, networkState){
            console.log("went offline");
            $cordovaToast.showLongBottom('There is no internet connection');
          });
 
        }
        else {
 
          window.addEventListener("online", function(e) {
            console.log("went online");
          }, false);    
 
          window.addEventListener("offline", function(e) {
            console.log("went offline");
          }, false);  
        }       
    }
  };
})

.factory('AppDB', function ($cordovaSQLite,$q,$http,DBA){
  
  var  storageList = [];
  var  bookingsList_sent = [];
  var  bookingsList_unsent = [];

      return {

        init: function(){

          DBA.query("CREATE TABLE IF NOT EXISTS Storages " +
                "( id     INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "  route    TEXT, " +
                "  data     TEXT);")
                .then(function(result) {
                    return true;
                });

          return DBA.query("CREATE TABLE IF NOT EXISTS Bookings (id INTEGER PRIMARY KEY AUTOINCREMENT, message TEXT,amount INT, sent  Boolean);")
                .then(function(result) {
                    return true;
                });

          // DBA.query("SELECT * FROM Bookings ORDER BY id DESC")
          //       .then(function(res) {
          //           if (res.rows.length > 0) {

          //                 for (var i = 0; i < res.rows.length; i++)
          //                 {
          //                      dataItem = {
          //                           id           : res.rows.item(i).id,
          //                           message : res.rows.item(i).message,
          //                           amount  : res.rows.item(i).amount,
          //                           sent       : (res.rows.item(i).sent == "true")
          //                       };
          //                     if(res.rows.item(i).sent == "true")bookingsList_sent.push(dataItem);       
          //                     else bookingsList_unsent.push(dataItem);
          //                 }
                          
          //           }
          //       });
        },
        load_storage : function(){

          return DBA.query("SELECT * FROM Storages ORDER BY id DESC;")
                .then(function(res) {
                    if (res.rows.length > 0) {

                          for (var i = 0; i < res.rows.length; i++)
                          {
                              var dataItem = {
                                  id          : res.rows.item(i).id,
                                  route   : res.rows.item(i).route,
                                  data    : res.rows.item(i).data     
                              };
                              storageList.push(dataItem);
                          }
                          
                    }
                });
        },
        delete_booking : function(booking){
                      // Execute query statement from query variable.
                       $cordovaSQLite.execute(db, 'DELETE FROM Bookings WHERE id = (?)',[booking.id]).then(function (res)
                       {
                           bookingsList_unsent.splice(bookingsList_unsent.indexOf(booking), 1);
                       });
        },
        save_booking : function(booking){
                      // Execute query statement from query variable.
                       $cordovaSQLite.execute(db, 'INSERT INTO Bookings (message,amount,sent) VALUES (?,?,?)',[booking.message,booking.amount,booking.sent]).then(function (res)
                       {
                           var dataItem = {
                               id      : res.insertId,
                               message   : booking.message,
                               amount   : booking.amount,
                               sent    : (booking.sent == "true")  
                           };
                           bookingsList_unsent.push(dataItem);
                       });
        },
        unsent_booking : function(){
            return bookingsList_unsent;
        },
        sent_booking: function(){
            return bookingsList_sent;
        },
        get: function(route){

              for (var i = 0; i < storageList.length; i++)
              {
                    if (storageList[i].route === route)
                    {
                        return storageList[i];
                    }
              }
        },
        // To get data to sqlite.
        // It will receive route from controller then insert it into sqlite.
        addorupdate: function (storage,update)
        {

            // Execute SELECT statement to load message from database.
            $cordovaSQLite.execute(db, 'SELECT * FROM Storages WHERE route=(?)',[storage.route])
                .then(
                    function(res) {

                    if (res.rows.length === 0) {
                       
                       // Execute query statement from query variable.
                       $cordovaSQLite.execute(db, 'INSERT INTO Storages (route,data) VALUES (?,?)',[storage.route,storage.data]).then(function (res)
                       {
                           var dataItem = {
                               id      : res.insertId,
                               route   : storage.route,
                               data    : storage.data      
                           };
                           storageList.push(dataItem);
                       });

                    }
                    else if(update === true){
                        // Execute query statement from query variable.
                        $cordovaSQLite.execute(db, 'UPDATE Storages SET data=(?) WHERE route=(?)',[storage.data,storage.route]).then(function (result)
                        {

                                  for (var i = 0; i < storageList.length; i++)
                                  {
                                      if (storageList[i].id === res.rows.item(0).id)
                                      {
                                          storageList[i].route = storage.route;
                                          storageList[i].data = storage.data;
                                      }
                                  }
                        });

                    }
                },
                function(error) {
                    console.log("DB Load Error!");
                }
            );
        }// End get data to sqlite.

    };

})

// Factory for node-pushserver (running locally in this case), if you are using other push notifications server you need to change this
.factory('NodePushServer', function ($http){
  // Configure push notifications server address
  //    - If you are running a local push notifications server you can test this by setting the local IP (on mac run: ipconfig getifaddr en1)
  var push_server_address = "http://192.168.1.207:8000";

  return {
    // Stores the device token in a db using node-pushserver
    // type:  Platform type (ios, android etc)
    storeDeviceToken: function(type, regId) {
      // Create a random userid to store with it
      var user = {
        user: 'user' + Math.floor((Math.random() * 10000000) + 1),
        type: type,
        token: regId
      };
      console.log("Post token for registered device with data " + JSON.stringify(user));

      $http.post(push_server_address+'/subscribe', JSON.stringify(user))
      .success(function (data, status) {
        console.log("Token stored, device is successfully subscribed to receive push notifications.");
      })
      .error(function (data, status) {
        console.log("Error storing device token." + data + " " + status);
      });
    },
    // CURRENTLY NOT USED!
    // Removes the device token from the db via node-pushserver API unsubscribe (running locally in this case).
    // If you registered the same device with different userids, *ALL* will be removed. (It's recommended to register each
    // time the app opens which this currently does. However in many cases you will always receive the same device token as
    // previously so multiple userids will be created with the same token unless you add code to check).
    removeDeviceToken: function(token) {
      var tkn = {"token": token};
      $http.post(push_server_address+'/unsubscribe', JSON.stringify(tkn))
      .success(function (data, status) {
        console.log("Token removed, device is successfully unsubscribed and will not receive push notifications.");
      })
      .error(function (data, status) {
        console.log("Error removing device token." + data + " " + status);
      });
    }
  };
});

})();
